Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.Net
Imports Bdt.Client.Sockets
Imports Bdt.Client.Socks
Imports Bdt.Shared.Runtime
Imports Bdt.Shared.Console
Imports Bdt.Shared.Service
Imports Bdt.Shared.Protocol
Imports Bdt.Shared.Logs
Imports Bdt.Shared.Request
Imports Bdt.Shared.Response
#End Region

Namespace Bdt.Client.Runtime

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Programme ct client du tunnel de communication
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class BdtClient
        Inherits Program

#Region " Constantes "
        ' Les constantes du fichier de configuration lies aux forwards
        Protected Const CFG_FORWARD_TEMPLATE As String = "forward/port{0}@"
        Protected Const CFG_SOCKS As String = "socks@"
        Protected Const CFG_LOCALPORT As String = "localport"
        Protected Const CFG_PORT As String = "port"
        Protected Const CFG_SHARED As String = "shared"
        Protected Const CFG_ADDRESS As String = "address"
        Protected Const CFG_ENABLED As String = "enabled"

        ' Les constantes du fichier de configuration lies au proxy
        Protected Const CFG_PROXY As String = "proxy"
        Protected Const CFG_PROXY_AUTH As String = CFG_PROXY & "/authentification@"
        Protected Const CFG_PROXY_CONFIG As String = CFG_PROXY & "/configuration@"
        Protected Const CFG_PROXY_ENABLED As String = CFG_PROXY & "@enabled"

        ' Configuration du proxy
        Protected Const CFG_PROXY_CONFIG_AUTO As String = CFG_PROXY_CONFIG & "auto"
        Protected Const CFG_PROXY_ADDRESS As String = CFG_PROXY_CONFIG & "address"
        Protected Const CFG_PROXY_PORT As String = CFG_PROXY_CONFIG & "port"

        ' Authentification du proxy
        Protected Const CFG_PROXY_AUTH_AUTO As String = CFG_PROXY_AUTH & "auto"
        Protected Const CFG_PROXY_USERNAME As String = CFG_PROXY_AUTH & "username"
        Protected Const CFG_PROXY_PASSWORD As String = CFG_PROXY_AUTH & "password"
        Protected Const CFG_PROXY_DOMAIN As String = CFG_PROXY_AUTH & "domain"

        ' Authentification service
        Protected Const CFG_SERVICE_USERNAME As String = GenericProtocol.CFG_SERVICE & "username"
        Protected Const CFG_SERVICE_PASSWORD As String = GenericProtocol.CFG_SERVICE & "password"
#End Region

#Region " Attributs "
        ' Les forwards de ports et socks, indexs par port local, en valeur le TcpServer associ au port
        Protected m_servers As New Hashtable
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Point d'entre du programme BdtClient
        ''' </summary>
        ''' <param name="args">les arguments de la ligne de commande</param>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub Main(ByVal args As String())
            Dim client As New BdtClient
            client.Start(args)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Mise en place des ports  l'coute pour les forwards via le tunnel
        ''' </summary>
        ''' <param name="tunnel">le tunnel cre avec le serveur</param>
        ''' <param name="uid">le user id courant</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub InitializeForwards(ByVal tunnel As ITunnel, ByVal uid As Integer)
            For i As Integer = IPEndPoint.MinPort To IPEndPoint.MaxPort
                Dim prefix As String = String.Format(CFG_FORWARD_TEMPLATE, i)
                If Configuration.ValueBool(prefix & CFG_ENABLED, False) Then
                    Dim port As Integer = Configuration.ValueInt(prefix & CFG_PORT)
                    Dim [shared] As Boolean = Configuration.ValueBool(prefix & CFG_SHARED)
                    Dim address As String = Configuration.Value(prefix & CFG_ADDRESS)
                    If m_servers.ContainsKey(i) Then
                        Log(String.Format("Le port {0} est dj occup, forward vers {1}:{2} annul", i, address, port), ILogger.ESeverity.WARN)
                    Else
                        Dim server As New GatewayServer(i, [shared], port, address, tunnel, uid)
                        m_servers.Add(i, server)
                    End If
                End If
            Next
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Mise en place du serveur Socks
        ''' </summary>
        ''' <param name="tunnel">le tunnel cre avec le serveur</param>
        ''' <param name="uid">le user id courant</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub InitializeSocks(ByVal tunnel As ITunnel, ByVal uid As Integer)
            If Configuration.ValueBool(CFG_SOCKS & CFG_ENABLED, False) Then
                Dim port As Integer = Configuration.ValueInt(CFG_SOCKS & CFG_PORT)
                Dim [shared] As Boolean = Configuration.ValueBool(CFG_SOCKS & CFG_SHARED)
                If m_servers.ContainsKey(port) Then
                    Log(String.Format("Le port {0} est dj occup, Serveur Socks dsactiv", port), ILogger.ESeverity.WARN)
                Else
                    Dim socks As New SocksServer(port, [shared], tunnel, uid)
                    m_servers.Add(port, socks)
                End If
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Destruction des ports  l'coute pour les forwards via le tunnel
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Sub DisposeServers()
            For Each server As Bdt.Client.Sockets.TcpServer In m_servers.Values
                server.CloseServer()
            Next
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Configuration du proxy http
        ''' </summary>
        ''' <param name="protocol">le protocole  configurer</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub ConfigureProxy(ByVal protocol As GenericProtocol)
            If TypeOf (protocol) Is IProxyCompatible Then
                Dim proxyProtocol As IProxyCompatible = DirectCast(protocol, GenericHttpRemoting)
                Dim proxy As IWebProxy

                If Configuration.ValueBool(CFG_PROXY_ENABLED) Then
                    ' Configuration
                    If Configuration.ValueBool(CFG_PROXY_CONFIG_AUTO) Then
                        proxy = GlobalProxySelection.Select ' - Framework 1.1
                        'proxy = WebRequest.GetSystemWebProxy() ' - Framework 2.0 MAIS semble non fonctionnel
                    Else
                        proxy = New WebProxy(Configuration.Value(CFG_PROXY_ADDRESS), Configuration.ValueInt(CFG_PROXY_PORT))
                    End If

                    If Not IsNothing(proxy) Then
                        ' Authentification
                        If Configuration.ValueBool(CFG_PROXY_AUTH_AUTO) Then
                            proxy.Credentials = System.Net.CredentialCache.DefaultCredentials
                        Else
                            Dim netCreds As New NetworkCredential
                            netCreds.UserName = Configuration.Value(CFG_PROXY_USERNAME)
                            netCreds.Password = Configuration.Value(CFG_PROXY_PASSWORD)
                            netCreds.Domain = Configuration.Value(CFG_PROXY_DOMAIN)
                            proxy.Credentials = netCreds
                        End If
                    End If
                Else
                    proxy = GlobalProxySelection.GetEmptyWebProxy ' - Framework 1.1
                    'proxy = Nothing ' - Framework 2.0 MAIS semble non fonctionnel
                End If

                If TypeOf (proxy) Is WebProxy AndAlso IsNothing(DirectCast(proxy, WebProxy).Address) Then
                    proxy = GlobalProxySelection.GetEmptyWebProxy
                    'proxy = Nothing ' - Framework 2.0 MAIS semble non fonctionnel
                End If

                proxyProtocol.Proxy = proxy

                If Not IsNothing(proxy) AndAlso TypeOf (proxy) Is WebProxy Then
                    Log(String.Format("Connexion avec un proxy: {0}", DirectCast(proxy, WebProxy).Address, proxy), ILogger.ESeverity.INFO)
                Else
                    Log("Connexion directe sans proxy", ILogger.ESeverity.INFO)
                End If
            End If

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Saisie d'une information sur stdin
        ''' </summary>
        ''' <param name="msg">le message  afficher</param>
        ''' <param name="hide">pour masquer l'echo sur stdout (ex: mot de passe)</param>
        ''' <returns>le chaine saisie</returns>
        ''' -----------------------------------------------------------------------------
        Protected Function InputString(ByVal msg As String, Optional ByVal hide As Boolean = False) As String
            Dim stdHandle As IntPtr
            Dim oriMode As Integer
            Console.Write(String.Format("INPUT {0}", msg))

            If hide Then
                ' Dsactive l'echo sur la sortie standard
                stdHandle = ConsoleTools.GetStdHandle(ConsoleTools.CONSOLE_STANDARD_HANDLES.STD_INPUT_HANDLE)
                ConsoleTools.GetConsoleMode(stdHandle, oriMode)
                ConsoleTools.SetConsoleMode(stdHandle, oriMode Xor ConsoleTools.CONSOLE_INPUT_MODES.ENABLE_ECHO_INPUT)
            End If

            Dim result As String = Console.ReadLine()

            If hide Then
                ' Ractive l'echo sur la sortie standard
                ConsoleTools.SetConsoleMode(stdHandle, oriMode)
                Console.Out.WriteLine()
            End If

            Return result
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Saisie des informations d'authentification sur le proxy
        ''' </summary>
        ''' <param name="proxyProtocol">le protocol IProxyCompatible  alterer</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub InputProxyCredentials(ByVal proxyProtocol As IProxyCompatible)
            Log("Une authentification sur le proxy est ncessaire", ILogger.ESeverity.INFO)
            Dim username As String = InputString("Entrez l'utilisateur: ")
            Dim password As String = InputString("Entrez le mot de passe (masqu): ", True)
            Dim domain As String = InputString("Entrez le domaine: ")

            proxyProtocol.Proxy.Credentials = New NetworkCredential(username, password, domain)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Traitement principal
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub Run()
            Try
                Log(String.Format("-=[ BoutDuTunnel Client v{0} - SLE ]=-", Me.GetType.Assembly.GetName.Version), ILogger.ESeverity.INFO)
                Log(String.Empty, ILogger.ESeverity.INFO)
                Protocol.ConfigureClient()
                ConfigureProxy(Protocol)

                Dim tunnel As ITunnel = Nothing
                Dim response As GenericResponse = Nothing
                Dim retry As Boolean

                ' Communication avec le tunnel. Dtection ventuelle d'une authentification incorrecte proxy
                Do
                    retry = False
                    Try
                        tunnel = Protocol.GetTunnel()
                        response = tunnel.Version
                    Catch ex As WebException
                        If TypeOf (ex.Response) Is HttpWebResponse AndAlso TypeOf (Protocol) Is IProxyCompatible Then
                            Dim proxyProtocol As IProxyCompatible = DirectCast(Protocol, IProxyCompatible)
                            Dim httpres As HttpWebResponse = DirectCast(ex.Response, HttpWebResponse)
                            If httpres.StatusCode = HttpStatusCode.ProxyAuthenticationRequired Then
                                InputProxyCredentials(proxyProtocol)
                                retry = True
                            End If
                        Else
                            Throw ex
                        End If
                    End Try
                Loop Until Not retry

                ' Dmarrage effectif du client, rcuperation de l'user-id
                If Not IsNothing(response) AndAlso Not IsNothing(tunnel) Then
                    If response.Message.IndexOf(Me.GetType.Assembly.GetName.Version.ToString) < 0 Then
                        Log("ATTENTION, les versions du client et du serveur ne sont pas en phase", ILogger.ESeverity.WARN)
                    End If
                    Log(response.Message, ILogger.ESeverity.INFO)
                    If response.Success Then
                        Dim username As String = Configuration.Value(CFG_SERVICE_USERNAME)
                        Dim password As String = Configuration.Value(CFG_SERVICE_PASSWORD)
                        Dim authResponse As AuthenticateResponse = tunnel.Authenticate(New AuthenticateRequest(username, password))
                        Log(authResponse.Message, ILogger.ESeverity.INFO)
                        If authResponse.Success Then
                            ' Mise En place des forwards
                            InitializeForwards(tunnel, authResponse.Uid())
                            ' Puis du serveur Socks
                            InitializeSocks(tunnel, authResponse.Uid())
                            Log("Client dmarr, appuyez sur une touche pour quitter...", ILogger.ESeverity.INFO)
                            Console.ReadLine()
                            ' Nettoyage
                            DisposeServers()
                        End If
                    End If
                End If
            Catch ex As Exception
                Log(ex.Message, ILogger.ESeverity.ERROR)
            End Try
        End Sub
#End Region

    End Class

End Namespace
