Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.Net
Imports System.Net.Sockets
Imports System.Threading
Imports Bdt.Shared.Logs
#End Region

Namespace Bdt.Client.Sockets

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Serveur TCP de base
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class TcpServer
        Inherits LoggedObject

#Region " Constantes "
        Public Const ACCEPT_POLLING_TIME As Integer = 50
#End Region

#Region " Attributs "
        Private m_listener As TcpListener
        Private m_mre As New ManualResetEvent(False)
        Private m_ip As IPAddress = IPAddress.Any
        Private m_port As Integer
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' L'ip d'coute
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Ip() As IPAddress
            Get
                Return m_ip
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le port d'coute
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Port() As Integer
            Get
                Return m_port
            End Get
        End Property
#End Region

#Region " Methodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur
        ''' </summary>
        ''' <param name="port">port local ct client</param>
        ''' <param name="shared">bind sur toutes les ip/ip locale</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal port As Integer, ByVal [shared] As Boolean)
            m_ip = DirectCast(IIf([shared], IPAddress.Any, IPAddress.Loopback), IPAddress)
            m_port = port

            m_listener = New TcpListener(m_ip, m_port)
            Dim thr As New Thread(AddressOf ServerThread)
            thr.Start()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Callback en cas de nouvelle connexion
        ''' </summary>
        ''' <param name="client">le socket client</param>
        ''' -----------------------------------------------------------------------------
        Protected MustOverride Sub OnNewConnection(ByVal client As TcpClient)

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fermeture de l'coute
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub CloseServer()
            m_mre.Set()
            m_listener.Stop()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Traitement principal du thread
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Sub ServerThread()
            Try
                m_listener.Start()
                While (Not m_mre.WaitOne(ACCEPT_POLLING_TIME, False))
                    Try
                        Dim client As TcpClient = m_listener.AcceptTcpClient()
                        OnNewConnection(client)
                    Catch ex As SocketException
                        If ex.ErrorCode <> 10004 Then
                            Log(ex.Message, ILogger.ESeverity.ERROR)
                        End If
                    Catch ex As Exception
                        Log(ex.Message, ILogger.ESeverity.ERROR)
                    End Try
                End While
            Catch ex As Exception
                Log(ex.Message, ILogger.ESeverity.ERROR)
            End Try
        End Sub
#End Region

    End Class

End Namespace
