Option Explicit On
Option Strict On

#Region " Inclusions "
Imports System.Net.Sockets
Imports Bdt.Shared.Logs
#End Region

Namespace Bdt.Client.Socks

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gestionnaire gnrique Socks
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class GenericSocksHandler
        Inherits LoggedObject

#Region " Constantes "
        ' La taille du buffer d'IO
        Public Const BUFFER_SIZE As Integer = 8192
#End Region

#Region " Attributs "
        Private m_version As Integer
        Private m_command As Integer
        Private m_address As String
        Private m_remoteport As Integer
        Private m_buffer() As Byte
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le handler est-il adapt  la requte?
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public MustOverride ReadOnly Property IsHandled() As Boolean

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Les donnes de rponse
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public MustOverride ReadOnly Property Reply() As Byte()

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' La version de la requte socks
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Property Version() As Integer
            Get
                Return m_version
            End Get
            Set(ByVal value As Integer)
                m_version = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' La commande de la requte socks
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Property Command() As Integer
            Get
                Return m_command
            End Get
            Set(ByVal value As Integer)
                m_command = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le port distant
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property RemotePort() As Integer
            Get
                Return m_remoteport
            End Get
            Protected Set(ByVal value As Integer)
                m_remoteport = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' L'adresse distante
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property Address() As String
            Get
                Return m_address
            End Get
            Protected Set(ByVal value As String)
                m_address = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le buffer de la requte
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected ReadOnly Property Buffer() As Byte()
            Get
                Return m_buffer
            End Get
        End Property

#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur
        ''' </summary>
        ''' <param name="buffer"></param>
        ''' -----------------------------------------------------------------------------
        Protected Sub New(ByVal buffer() As Byte)
            m_buffer = buffer
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne un gestionnaire adapt  la requte
        ''' </summary>
        ''' <param name="client">le client TCP</param>
        ''' <returns>un gestionnaire adapt</returns>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetInstance(ByVal client As TcpClient) As GenericSocksHandler
            Dim buffer(BUFFER_SIZE) As Byte

            Dim stream As NetworkStream = client.GetStream
            Dim size As Integer = stream.Read(buffer, 0, BUFFER_SIZE)
            ReDim Preserve buffer(size - 1)

            If size < 3 Then
                Throw New ArgumentException("Echange incorrect avec le gestionnaire Socks")
            End If

            Dim result As GenericSocksHandler
            result = New Socks4Handler(buffer)
            If Not result.IsHandled Then
                result = New Socks4AHandler(buffer)
                If Not result.IsHandled Then
                    result = New Socks5Handler(client, buffer)
                    If Not result.IsHandled Then
                        Throw New ArgumentException("Aucun gestionnaire Socks adapt pour la requte")
                    End If
                End If
            End If

            Dim reply As Byte() = result.Reply
            client.GetStream.Write(reply, 0, reply.Length)

            Return result
        End Function
#End Region

    End Class

End Namespace

