Option Explicit On
Option Strict On

Namespace Bdt.Client.Socks

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gestionnaire Socks v4 (sans DNS)
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class Socks4Handler
        Inherits GenericSocksHandler

#Region " Constantes "
        Public Const SOCKS4_REPLY_VERSION As Integer = 0
        Public Const SOCKS4_OK As Integer = 90 ' Requte accepte
        Public Const SOCKS4_KO As Integer = 91 ' Requte refuse
        Public Const SOCKS4_CONNECT_COMMAND As Integer = 1  ' commande CONNECT
        Public Const SOCKS4_BIND_COMMAND As Integer = 2  ' commande BIND (non supporte)
        Public Const REPLY_SIZE As Integer = 8  ' octets de rponse
#End Region

#Region " Attributs "
        Protected m_reply(REPLY_SIZE - 1) As Byte
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le handler est-il adapt  la requte?
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides ReadOnly Property IsHandled() As Boolean
            Get
                ' Prparation pessimiste de la rponse
                m_reply(0) = SOCKS4_REPLY_VERSION
                m_reply(1) = SOCKS4_KO
                Array.Clear(m_reply, 2, 6)

                If (Version <> 4) Then Return False

                ' Teste pour basic Socks4 (pas Socks4a)
                If ((Buffer(4) <> 0) OrElse (Buffer(5) <> 0) OrElse (Buffer(6) <> 0)) Then
                    If (Command <> SOCKS4_BIND_COMMAND) Then
                        RemotePort = 256 * Convert.ToInt32(Buffer(2)) + Convert.ToInt32(Buffer(3))
                        Address = Buffer(4) & "." & Buffer(5) & "." & Buffer(6) & "." & Buffer(7)
                        ' Prparation de la rponse
                        m_reply(1) = SOCKS4_OK
                        Array.Copy(Buffer, 2, m_reply, 2, 6)
                        Log("Requte Socks4 supporte", [Shared].Logs.ILogger.ESeverity.DEBUG)
                        Return True
                    Else
                        ' Socks4 BIND 
                        Log("Requte Socks4 BIND non supporte", [Shared].Logs.ILogger.ESeverity.WARN)
                    End If
                End If
                Return False
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Les donnes de rponse
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides ReadOnly Property Reply() As Byte()
            Get
                Return m_reply
            End Get
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur
        ''' </summary>
        ''' <param name="buffer"></param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal buffer() As Byte)
            MyBase.New(buffer)
            Version = buffer(0)
            Command = buffer(1)
        End Sub
#End Region

    End Class
End Namespace
