Option Explicit On
Option Strict On

#Region " Inclusions "
Imports System.Net
Imports System.Net.Sockets
Imports System.Threading
Imports Bdt.Client.Sockets
Imports Bdt.Shared.Logs
Imports Bdt.Shared.Service
#End Region

Namespace Bdt.Client.Socks

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Serveur Socks avec gestion v4, v4A, v5
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class SocksServer
        Inherits TcpServer

#Region " Attributs "
        Protected m_tunnel As ITunnel
        Protected m_uid As Integer
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur
        ''' </summary>
        ''' <param name="port">port local ct client</param>
        ''' <param name="shared">bind sur toutes les ip/ip locale</param>
        ''' <param name="tunnel">tunnel de communication</param>
        ''' <param name="uid">user-id</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal port As Integer, ByVal [shared] As Boolean, ByVal tunnel As ITunnel, ByVal uid As Integer)
            MyBase.New(port, [shared])
            Log(String.Format("Serveur Socks  l'coute {0}:{1}", Ip, port), ILogger.ESeverity.DEBUG)

            m_tunnel = tunnel
            m_uid = uid
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Callback en cas de nouvelle connexion
        ''' </summary>
        ''' <param name="client">le socket client</param>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub OnNewConnection(ByVal client As System.Net.Sockets.TcpClient)
            Dim handler As GenericSocksHandler = Nothing
            Try
                handler = GenericSocksHandler.GetInstance(client)
            Catch ex As Exception
                Log(ex.Message, ILogger.ESeverity.ERROR)
            End Try
            If Not IsNothing(handler) Then
                Dim gateway As New Gateway(client, m_tunnel, m_uid, handler.Address, handler.RemotePort)
            End If
        End Sub

#End Region

    End Class

End Namespace
