Option Explicit On 
Option Strict On

Namespace Bdt.Shared.Configuration

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Reprsente une source de configuration gnrique. Permet de servir de base pour l'laboration
    ''' d'autres sources.
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class BaseConfig
        Implements IComparable

#Region " Constantes "
        Public Const SOURCE_PATH_SEPARATOR As String = "/"
        Public Const SOURCE_ITEM_ATTRIBUTE As String = "@"
        Public Const SOURCE_ITEM_EQUALS As String = "="
        Public Const SOURCE_SCRAMBLED_START As String = "["
        Public Const SOURCE_SCRAMBLED_END As String = "]"
#End Region

#Region " Attributs "
        Protected m_values As New SortedList               ' Les elements classs par code
        Protected m_priority As Integer = 0                ' La priorit de cette source
#End Region

#Region " Proprits "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne/Fixe la priorit de la source
        ''' </summary>
        ''' <returns>la priorit de la source</returns>
        ''' -----------------------------------------------------------------------------
        Public Property Priority() As Integer
            Get
                Return m_priority
            End Get
            Set(ByVal Value As Integer)
                m_priority = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne/Fixe la priorit de la source
        ''' </summary>
        ''' <returns>la priorit de la source</returns>
        ''' -----------------------------------------------------------------------------
        Public Property Value(ByVal code As String, Optional ByVal defaultValue As String = Nothing) As String
            Get
                If m_values.ContainsKey(code) Then
                    Return DirectCast(m_values(code), String)
                Else
                    Return defaultValue
                End If
            End Get
            Set(ByVal Value As String)
                If m_values.ContainsKey(code) Then
                    m_values(code) = Value
                Else
                    m_values.Add(code, Value)
                End If
            End Set
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur d'une source avec un dcrypteur de donnes optionnel
        ''' Les valeurs entre SOURCE_SCRAMBLED_START et SOURCE_SCRAMBLED_END seront
        ''' automatiquement dcryptes
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Sub New(Optional ByVal priority As Integer = 0)
            Me.Priority = priority
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Force le rechargement de la source de donne
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public MustOverride Sub Rehash()

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Concatne tous les lments depuis cette source
        ''' </summary>
        ''' <returns>le format de chaque ligne est classe,priorit,code,valeur</returns>
        ''' -----------------------------------------------------------------------------
        Public NotOverridable Overrides Function ToString() As String
            ToString = String.Empty

            For Each key As String In m_values.Keys
                ToString += "   <" + Me.GetType.Name + "(" & Priority & ")" + "> [" + key + "] " + SOURCE_ITEM_EQUALS + " [" + Value(key) + "]" + vbCrLf
            Next
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Comparateur par priorit
        ''' </summary>
        ''' <param name="obj">la config  comparer</param>
        ''' <returns>voir IComparable.CompareTo</returns>
        ''' -----------------------------------------------------------------------------
        Public Function CompareTo(ByVal obj As Object) As Integer Implements System.IComparable.CompareTo
            If TypeOf (obj) Is BaseConfig Then
                Return Me.Priority - DirectCast(obj, BaseConfig).Priority
            Else
                Return 0
            End If
        End Function
#End Region

    End Class
End Namespace
