Option Explicit On 
Option Strict On

Namespace Bdt.Shared.Configuration

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Reprsente une source de configuration base sur une ligne de commande
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public NotInheritable Class StringConfig
        Inherits BaseConfig

#Region " Attributs "
        Protected _Args() As String 'Les arguments de la ligne de commande
#End Region

#Region " Proprits "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne/Fixe les arguments de la ligne de commande
        ''' </summary>
        ''' <returns>les arguments de la ligne de commande</returns>
        ''' -----------------------------------------------------------------------------
        Public Property Args() As String()
            Get
                Return _Args
            End Get
            Set(ByVal Value() As String)
                _Args = Value
            End Set
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Cration d'une source de donne base sur la ligne de commande
        ''' </summary>
        ''' <param name="args">les arguments de la ligne de commande</param>
        ''' <param name="priority">la priorit de cette source (la plus basse=prioritaire)</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal args() As String, Optional ByVal priority As Integer = 0)
            MyBase.New(priority)
            Me.Args = args
            Rehash()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Force le rechargement de la source de donne
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub Rehash()
            For Each arg As String In Args
                Dim equalIndex As Integer = arg.IndexOf(SOURCE_ITEM_EQUALS)
                If (equalIndex >= 0) AndAlso equalIndex + 1 < arg.Length Then
                    Me.Value(arg.Substring(0, equalIndex)) = arg.Substring(equalIndex + 1)
                End If
            Next
        End Sub
#End Region

    End Class

End Namespace

