Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.Xml
#End Region

Namespace Bdt.Shared.Configuration

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Reprsente une source de configuration base sur un fichier XML
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public NotInheritable Class XMLConfig
        Inherits BaseConfig

#Region " Attributs "
        Protected _Filename As String = "" 'Le nom du fichier XML associ  la source
#End Region

#Region " Proprits "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne/Fixe le nom du fichier XML associ  la source
        ''' </summary>
        ''' <returns>le nom du fichier XML associ  la source</returns>
        ''' -----------------------------------------------------------------------------
        Public Property FileName() As String
            Get
                Return _Filename
            End Get
            Set(ByVal Value As String)
                _Filename = Value
            End Set
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Cration d'une source de donne base sur un fichier XML
        ''' </summary>
        ''' <param name="filename">le nom du fichier XML  lier</param>
        ''' <param name="priority">la priorit de cette source (la plus basse=prioritaire)</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal filename As String, Optional ByVal priority As Integer = 0)
            MyBase.New(priority)
            Me.FileName = filename
            Rehash()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Parsing des noeuds XML
        ''' </summary>
        ''' <param name="path">le chemin courant</param>
        ''' <param name="node">le noeud en cours</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub ParseNode(ByVal path As String, ByVal node As XmlNode)
            For Each subnode As XmlNode In node.ChildNodes

                If Not IsNothing(subnode.Attributes) Then
                    For Each attr As XmlAttribute In subnode.Attributes
                        If (attr.Value <> String.Empty) Then
                            Me.Value(path + subnode.Name + SOURCE_ITEM_ATTRIBUTE + attr.Name) = attr.Value
                        End If
                    Next
                    If (subnode.InnerText <> String.Empty) Then
                        Me.Value(path + subnode.Name) = subnode.InnerText
                    End If
                End If

                If (subnode.HasChildNodes) AndAlso (subnode.ChildNodes(0).NodeType = XmlNodeType.Element Or subnode.ChildNodes(0).NodeType = XmlNodeType.Comment) Then
                    ' Chemin
                    ParseNode(path + subnode.Name + SOURCE_PATH_SEPARATOR, subnode)
                End If
            Next
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Force le rechargement de la source de donne
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub Rehash()
            Dim docXML As New XmlDocument
            docXML.Load(FileName)
            ParseNode(String.Empty, docXML.DocumentElement)
        End Sub
#End Region

    End Class

End Namespace

