Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports Bdt.Shared.Configuration
Imports System.IO
#End Region

Namespace Bdt.Shared.Logs

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gnration des logs dans un fichier
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class FileLogger
        Inherits BaseLogger

#Region " Constantes "
        Public Const CONFIG_APPEND As String = "append"
        Public Const CONFIG_FILENAME As String = "filename"
#End Region

#Region " Attributs "
        Protected m_filename As String = Nothing
        Protected m_append As Boolean = False
#End Region

#Region " Proprits "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne le nom du fichier utilis pour l'criture des logs
        ''' </summary>
        ''' <returns>le nom du fichier utilis pour l'criture des logs</returns>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Filename() As String
            Get
                Return m_filename
            End Get
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur pour un log  partir des donnes fournies dans une configuration
        ''' </summary>
        ''' <param name="prefix">le prefixe dans la configuration ex: application/log</param>
        ''' <param name="config">la configuration pour la lecture des parametres</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal prefix As String, ByVal config As ConfigPackage)
            MyBase.New(Nothing, prefix, config)
            m_filename = config.Value(prefix & BaseConfig.SOURCE_ITEM_ATTRIBUTE & CONFIG_FILENAME, m_filename)
            m_append = config.ValueBool(prefix & BaseConfig.SOURCE_ITEM_ATTRIBUTE & CONFIG_APPEND, m_append)
            If Enabled Then
                m_writer = New StreamWriter(m_filename, m_append, System.Text.Encoding.Default)
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur pour un log
        ''' </summary>
        ''' <param name="filename">le nom du fichier dans lequel crire</param>
        ''' <param name="append">si false la fichier sera cras</param>
        ''' <param name="dateFormat">le format des dates de timestamp</param>
        ''' <param name="filter">le niveau de filtrage pour la sortie des logs</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal filename As String, Optional ByVal append As Boolean = False, Optional ByVal dateFormat As String = DEFAULT_DATE_FORMAT, Optional ByVal filter As ILogger.ESeverity = ILogger.ESeverity.DEBUG)
            MyBase.New(New StreamWriter(filename, append, System.Text.Encoding.Default), dateFormat, filter)
            m_filename = filename
            m_append = append
        End Sub
#End Region

    End Class

End Namespace
