Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.ComponentModel
#End Region

Namespace Bdt.Shared.Logs

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Classe de base pour un objet utilisant un flux de log
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    <Serializable(), TypeConverter(GetType(ExpandableObjectConverter))> _
    Public Class LoggedObject
        Implements ILogger

#Region " Attributs "
        Protected Shared m_globalLogger As BaseLogger = Nothing
        Protected m_logger As BaseLogger = Nothing
#End Region

#Region " Proprits "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fixe/retourne le loggueur assoc  cet objet
        ''' </summary>
        ''' <returns>le loggueur assoc  cet objet</returns>
        ''' -----------------------------------------------------------------------------
        Public Property Logger() As BaseLogger
            Get
                Return m_logger
            End Get
            Set(ByVal Value As BaseLogger)
                m_logger = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fixe/retourne le loggueur assoc  tous les objets drivs.
        ''' </summary>
        ''' <returns>le loggueur assoc  tous les objets drivs</returns>
        ''' -----------------------------------------------------------------------------
        Public Shared Property GlobalLogger() As BaseLogger
            Get
                Return m_globalLogger
            End Get
            Set(ByVal Value As BaseLogger)
                m_globalLogger = Value
            End Set
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ecriture d'une entre de log. Ne sera pas prise en compte si le log est inactif
        ''' ou si le filtre l'impose
        ''' </summary>
        ''' <param name="message">le message  logger</param>
        ''' <param name="severity">la svrit</param>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub Log(ByVal message As String, ByVal severity As ILogger.ESeverity)
            Log(Me, message, severity)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ecriture d'une entre de log. Ne sera pas prise en compte si le log est inactif
        ''' ou si le filtre l'impose
        ''' </summary>
        ''' <param name="message">le message  logger</param>
        ''' <param name="severity">la svrit</param>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub Log(ByVal sender As Object, ByVal message As String, ByVal severity As ILogger.ESeverity) Implements ILogger.Log
            If Not IsNothing(m_logger) Then
                m_logger.Log(sender, message, severity)
            End If
            If Not IsNothing(m_globalLogger) Then
                m_globalLogger.Log(sender, message, severity)
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fermeture du logger
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub Close() Implements ILogger.Close
            If Not IsNothing(m_logger) Then
                m_logger.Close()
            End If
        End Sub
#End Region

    End Class

End Namespace
