Option Explicit On 
Option Strict On

Namespace Bdt.Shared.Logs

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Classe permettant la propagation de logs sur de multiples flux
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public NotInheritable Class MultiLogger
        Inherits BaseLogger

#Region " Attributs "
        Protected _Loggers As New ArrayList
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur pour un flux multiple
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub New()
            MyBase.New(Nothing)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ajoute un logger  ce flux multiple
        ''' </summary>
        ''' <param name="logger"></param>
        ''' -----------------------------------------------------------------------------
        Public Sub AddLogger(ByVal logger As ILogger)
            _Loggers.Add(logger)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Supprime le logger de ce flux multiple
        ''' </summary>
        ''' <param name="logger"></param>
        ''' -----------------------------------------------------------------------------
        Public Sub RemoveLogger(ByVal logger As ILogger)
            _Loggers.Remove(logger)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ecriture d'une entre de log. Ne sera pas prise en compte si le log est inactif
        ''' ou si le filtre l'impose. L'criture sera propage  tous les flux associs
        ''' </summary>
        ''' <param name="sender">l'emetteur</param>
        ''' <param name="message">le message  logger</param>
        ''' <param name="severity">la svrit</param>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub Log(ByVal sender As Object, ByVal message As String, ByVal severity As ILogger.ESeverity)
            If Enabled Then
                For Each logger As ILogger In _Loggers
                    logger.Log(sender, message, severity)
                Next
            End If
        End Sub
#End Region

    End Class

End Namespace