Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.IO
Imports System.Text
Imports Bdt.Shared.Configuration
#End Region

Namespace Bdt.Shared.Logs

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gnration des logs dans une chaine
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public NotInheritable Class StringLogger
        Inherits BaseLogger

#Region " Attributs "
        Protected _LastLine As String = String.Empty
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne le texte complet logg dans cet objet
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Text() As String
            Get
                Return DirectCast(m_writer, StringWriter).GetStringBuilder.ToString
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne la dernire ligne de log
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property LastLine() As String
            Get
                Return _LastLine
            End Get
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur pour un log  partir des donnes fournies dans une configuration
        ''' </summary>
        ''' <param name="prefix">le prefixe dans la configuration ex: application/log</param>
        ''' <param name="config">la configuration pour la lecture des parametres</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal prefix As String, ByVal config As ConfigPackage)
            MyBase.New(New StringWriter, prefix, config)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur pour un log
        ''' </summary>
        ''' <param name="dateFormat">le format des dates de timestamp</param>
        ''' <param name="filter">le niveau de filtrage pour la sortie des logs</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(Optional ByVal dateFormat As String = DEFAULT_DATE_FORMAT, Optional ByVal filter As ILogger.ESeverity = ILogger.ESeverity.DEBUG)
            MyBase.New(New StringWriter, dateFormat, filter)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ecriture d'une entre de log. Ne sera pas prise en compte si le log est inactif
        ''' ou si le filtre l'impose
        ''' </summary>
        ''' <param name="sender">l'emetteur</param>
        ''' <param name="message">le message  logger</param>
        ''' <param name="severity">la svrit</param>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub Log(ByVal sender As Object, ByVal message As String, ByVal severity As ILogger.ESeverity)
            Dim sb As StringBuilder = DirectCast(m_writer, StringWriter).GetStringBuilder
            Dim index As Integer = sb.Length
            MyBase.Log(sender, message, severity)
            If (m_enabled) AndAlso (severity >= m_filter) Then
                _LastLine = sb.ToString(index, sb.Length - index)
            End If
        End Sub
#End Region

    End Class

End Namespace