Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports Bdt.Shared.Configuration
Imports Bdt.Shared.Logs
Imports Bdt.Shared.Service
#End Region

Namespace Bdt.Shared.Protocol

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Classe gnrique pour un protocole de communication
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class GenericProtocol
        Inherits LoggedObject

#Region " Constantes "
        Public Const CFG_SERVICE As String = "service@"
        Public Const CFG_PROTOCOL As String = "protocol"
        Public Const CFG_NAME As String = "name"
        Public Const CFG_PORT As String = "port"
        Public Const CFG_ADDRESS As String = "address"
#End Region

#Region " Attributs "
        Protected m_name As String
        Protected m_port As Integer
        Protected m_address As String
        Protected m_configuration As ConfigPackage
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le nom du serveur
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Name() As String
            Get
                Return m_name
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le port du serveur
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Port() As Integer
            Get
                Return m_port
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' L'adresse du serveur
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Address() As String
            Get
                Return m_address
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' La configuration propre au serveur ou au client
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Configuration() As ConfigPackage
            Get
                Return m_configuration
            End Get
        End Property
#End Region

#Region " Methodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Configuration ct serveur
        ''' </summary>
        ''' <param name="type">le type d'objet  rendre distant</param>
        ''' -----------------------------------------------------------------------------
        Public MustOverride Sub ConfigureServer(ByVal type As Type)

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Configuration ct client
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public MustOverride Sub ConfigureClient()

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne une instance de tunnel
        ''' </summary>
        ''' <returns>une instance de tunnel</returns>
        ''' -----------------------------------------------------------------------------
        Public MustOverride Function GetTunnel() As ITunnel

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Obtient un protocole adapt d'aprs la configuration
        ''' </summary>
        ''' <param name="config">la configuration contenant le type du protocole</param>
        ''' <returns>une instance adapte</returns>
        ''' -----------------------------------------------------------------------------
        Public Shared Function GetInstance(ByVal config As ConfigPackage) As GenericProtocol
            Dim protoStr As String = config.Value(CFG_SERVICE & CFG_PROTOCOL)
            Dim protoObj As GenericProtocol = DirectCast(GetType(GenericProtocol).Assembly.CreateInstance(protoStr), GenericProtocol)
            protoObj.m_name = config.Value(CFG_SERVICE & CFG_NAME)
            protoObj.m_port = config.ValueInt(CFG_SERVICE & CFG_PORT)
            protoObj.m_address = config.Value(CFG_SERVICE & CFG_ADDRESS)
            protoObj.m_configuration = config
            Return protoObj
        End Function
#End Region

    End Class

End Namespace
