Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.Runtime.Remoting
Imports System.Runtime.Remoting.Channels
Imports Bdt.Shared.Service
#End Region

Namespace Bdt.Shared.Protocol

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Protocole de communication bas sur le remoting .NET
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class GenericRemoting
        Inherits GenericProtocol

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le canal de communication ct serveur
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected MustOverride ReadOnly Property ServerChannel() As IChannel

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le canal de communication ct client
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public MustOverride ReadOnly Property ClientChannel() As IChannel

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' L'URL ncessaire pour se connecter au serveur
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected MustOverride ReadOnly Property ServerURL() As String
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Configuration ct client
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub ConfigureClient()
            Log(String.Format("Configuration {0} => {1}", Me.GetType.Name, ServerURL), Logs.ILogger.ESeverity.INFO)
            ChannelServices.RegisterChannel(ClientChannel, False)
            Log("Configuration termine", Logs.ILogger.ESeverity.INFO)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Configuration ct serveur
        ''' </summary>
        ''' <param name="type">le type d'objet  rendre distant</param>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub ConfigureServer(ByVal type As Type)
            Log(String.Format("Configuration {0} port {1}", Me.GetType.Name, Port), Logs.ILogger.ESeverity.INFO)
            ChannelServices.RegisterChannel(ServerChannel, False)
            Dim wks As New WellKnownServiceTypeEntry(type, Name, WellKnownObjectMode.Singleton)
            RemotingConfiguration.RegisterWellKnownServiceType(wks)
            Log("Configuration termine", Logs.ILogger.ESeverity.INFO)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne une instance de tunnel
        ''' </summary>
        ''' <returns>une instance de tunnel</returns>
        ''' -----------------------------------------------------------------------------
        Public Overrides Function GetTunnel() As Service.ITunnel
            Return DirectCast(Activator.GetObject(GetType(ITunnel), ServerURL), ITunnel)
        End Function
#End Region

    End Class

End Namespace
