Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports Bdt.Shared.Logs
Imports Bdt.Shared.Configuration
Imports Bdt.Shared.Protocol
#End Region

Namespace Bdt.Shared.Runtime

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Une bauche de programme
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class Program
        Inherits LoggedObject

#Region " Constantes "
        Protected Const CFG_LOG As String = "logs/"
        Protected Const CFG_CONSOLE As String = CFG_LOG & "console"
        Protected Const CFG_FILE As String = CFG_LOG & "file"
#End Region

#Region " Attributs "
        Protected m_configuration As New ConfigPackage
        Protected m_protocol As GenericProtocol
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le fichier de configuration
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected ReadOnly Property ConfigFile() As String
            Get
                Return String.Format("{0}Cfg.xml", Me.GetType.Assembly.GetName.Name)
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Les donnes de configuration
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected ReadOnly Property Configuration() As ConfigPackage
            Get
                Return m_configuration
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le protocole de communication
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected ReadOnly Property Protocol() As GenericProtocol
            Get
                Return m_protocol
            End Get
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Chargement des donnes de configuration
        ''' </summary>
        ''' <param name="args">Arguments de la ligne de commande</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub LoadConfiguration(ByVal args As String())
            Dim ldcConfig As New StringConfig(args, 0)
            Dim xmlConfig As New XMLConfig(ConfigFile, 1)
            Configuration.AddSource(ldcConfig)
            Configuration.AddSource(xmlConfig)

            Dim log As New MultiLogger
            log.AddLogger(New ConsoleLogger(CFG_CONSOLE, Configuration))
            log.AddLogger(New FileLogger(CFG_FILE, Configuration))
            LoggedObject.GlobalLogger = log
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Traitement principal
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected MustOverride Sub Run()

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Dmarrage de l'application
        ''' </summary>
        ''' <param name="args">Arguments de la ligne de commande</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub Start(ByVal args As String())
            LoadConfiguration(args)
            m_protocol = GenericProtocol.GetInstance(Configuration)
            Run()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Encodeur simple par Xor pour un tableau d'octets
        ''' </summary>
        ''' <param name="bytes">Le tableau  encoder/dcoder (xor rversible)</param>
        ''' <param name="seed">La racine d'initialisation du gnrateur alatoire</param>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub RandomXorEncoder(ByRef bytes() As Byte, ByVal seed As Integer)
            Dim rnd As New Random(seed)
            If Not IsNothing(bytes) Then
                For i As Integer = 0 To bytes.Length - 1
                    bytes(i) = bytes(i) Xor Convert.ToByte(Math.Abs(rnd.Next Mod 256))
                Next
            End If
        End Sub
#End Region

    End Class

End Namespace
